@@ -52,10 +52,10 @@ npindexbw.formula <-
   }
 
 
-npindexbw.NULL <-
-  function(xdat = stop("training data xdat missing"),
-           ydat = stop("training data ydat missing"),
-           bws, ...){
+npindexbw.NULL <- function(xdat = stop("training data xdat missing"),
+                           ydat = stop("training data ydat missing"),
+                           bws,
+                           ...){
 
     xdat <- toFrame(xdat)
 
@@ -63,7 +63,8 @@ npindexbw.NULL <-
 
     tbw <- npindexbw.default(xdat = xdat,
                              ydat = ydat,
-                             bws = bws, ...)
+                             bws = bws,
+                             ...)
 
     ## clean up (possible) inconsistencies due to recursion ...
     mc <- match.call(expand.dots = FALSE)
@@ -81,9 +82,18 @@ npindexbw.NULL <-
 npindexbw.default <-
   function(xdat = stop("training data xdat missing"),
            ydat = stop("training data ydat missing"),
-           bws, bandwidth.compute = TRUE,
-           nmulti, random.seed, optim.method, optim.maxattempts,
-           optim.reltol, optim.abstol, optim.maxit, ...){
+           bws,
+           bandwidth.compute = TRUE,
+           nmulti,
+           random.seed,
+           optim.method,
+           optim.maxattempts,
+           optim.reltol,
+           optim.abstol,
+           optim.maxit,
+           ckertype,
+           ckerorder,
+           ...){
 
     xdat <- toFrame(xdat)
 
@@ -102,27 +112,57 @@ npindexbw.default <-
       stop(paste("manually specified 'bws' must be a numeric vector of length ncol(xdat)+1.",
                  "See documentation for details."))
 
-    tbw <- sibandwidth(beta = bws[1:ncol(xdat)],
-                       h = bws[ncol(xdat)+1], ...,
-                       nobs = dim(xdat)[1],
-                       xdati = untangle(xdat),
-                       ydati = untangle(data.frame(ydat)),
-                       xnames = names(xdat),
-                       ynames = deparse(substitute(ydat)),
-                       bandwidth = bws[ncol(xdat)+1],
-                       bandwidth.compute = bandwidth.compute)
+###
+    ## first grab dummy args for scbandwidth() and perform 'bootstrap'
+    ## bandwidth call
 
+    mc.names <- names(match.call(expand.dots = FALSE))
+    margs <- c("ckertype", "ckerorder")
 
+    m <- match(margs, mc.names, nomatch = 0)
+    any.m <- any(m != 0)
+
+    tbw <- eval(parse(text=paste("sibandwidth(beta = bws[1:ncol(xdat)]",
+                        ifelse(any.m, ",",""),
+                        paste(mc.names[m], ifelse(any.m,"=",""), mc.names[m], collapse=", "),
+                        ", h = bws[ncol(xdat)+1],",
+                        "...,",
+                        "nobs = dim(xdat)[1],",
+                        "xdati = untangle(xdat),",
+                        "ydati = untangle(data.frame(ydat)),",
+                        "xnames = names(xdat),",
+                        "ynames = deparse(substitute(ydat)),",
+                        "bandwidth = bws[ncol(xdat)+1],",
+                        "bandwidth.compute = bandwidth.compute)")))
+    
+#    tbw <- sibandwidth(beta = bws[1:ncol(xdat)],
+#                       h = bws[ncol(xdat)+1], ...,
+#                       nobs = dim(xdat)[1],
+#                       xdati = untangle(xdat),
+#                       ydati = untangle(data.frame(ydat)),
+#                       xnames = names(xdat),
+#                       ynames = deparse(substitute(ydat)),
+#                       bandwidth = bws[ncol(xdat)+1],
+#                       bandwidth.compute = bandwidth.compute)
+
+    ## next grab dummies for actual bandwidth selection and perform call
     if (tbw$method == "kleinspady" & !setequal(ydat,c(0,1))) 
       stop("Klein and Spady's estimator requires binary ydat with 0/1 values only")
 
     mc.names <- names(match.call(expand.dots = FALSE))
-    margs <- c("nmulti","random.seed", "optim.method", "optim.maxattempts",
-               "optim.reltol", "optim.abstol", "optim.maxit")
+    margs <- c("nmulti",
+               "random.seed",
+               "optim.method",
+               "optim.maxattempts",
+               "optim.reltol",
+               "optim.abstol",
+               "optim.maxit",
+               "ckertype",
+               "ckerorder")
     
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
-    
+
     if (bandwidth.compute)
       tbw <- eval(parse(text=paste("npindexbw.sibandwidth(xdat=xdat, ydat=ydat, bws=tbw",
                           ifelse(any.m, ",",""),
@@ -140,12 +180,17 @@ npindexbw.default <-
 npindexbw.sibandwidth <-
   function(xdat = stop("training data xdat missing"),
            ydat = stop("training data ydat missing"),
-           bws, bandwidth.compute = TRUE, nmulti, random.seed = 42,
+           bws,
+           bandwidth.compute = TRUE,
+           nmulti,
+           random.seed = 42,
            optim.method = c("Nelder-Mead", "BFGS", "CG"),
            optim.maxattempts = 10,
            optim.reltol = sqrt(.Machine$double.eps),
            optim.abstol = .Machine$double.eps,
            optim.maxit = 500,
+           ckertype = c("gaussian", "epanechnikov","uniform"), 
+           ckerorder = c(2,4,6,8),           
            ...){
 
     ## Save seed prior to setting
@@ -176,6 +221,18 @@ npindexbw.sibandwidth <-
                     "dimensionality is unnecessary."))
     }
 
+    ckertype = match.arg(ckertype)
+
+    if(missing(ckerorder))
+      ckerorder = 2
+    else if (ckertype == "uniform")
+      warning("ignoring kernel order specified with uniform kernel type")
+    else {
+      kord = eval(formals()$ckerorder) 
+      if (!any(kord == ckerorder))
+        stop("ckerorder must be one of ", paste(kord,collapse=" "))
+    }
+
     optim.method <- match.arg(optim.method)
     
     ## catch and destroy NA's
@@ -225,7 +282,7 @@ npindexbw.sibandwidth <-
 
         ## Next we define the sum of squared leave-one-out residuals
 
-        sum.squares.leave.one.out <- function(xdat,ydat,beta,h) {
+        sum.squares.leave.one.out <- function(xdat,ydat,beta,h,ckertype,ckerorder) {
 
           ## Normalize beta_1 = 1 hence multiply X by c(1,beta)
           
@@ -240,7 +297,10 @@ npindexbw.sibandwidth <-
                         tydat=W,
                         weights=W,
                         leave.one.out=TRUE,
-                        bandwidth.divide=TRUE,bws=c(h))$ksum
+                        bandwidth.divide=TRUE,
+                        bws=c(h),
+                        ckertype=ckertype,
+                        ckerorder=ckerorder)$ksum
 
           denom <- tww[2,2,]
           denom[which(denom == 0.0)] <- ichimuraFloor
@@ -255,7 +315,7 @@ npindexbw.sibandwidth <-
         ## return an infinite penalty for negative h
 
         if(h > 0) {
-          return(sum.squares.leave.one.out(xdat,ydat,beta,h))
+          return(sum.squares.leave.one.out(xdat=xdat,ydat=ydat,beta=beta,h=h,ckertype=ckertype,ckerorder=ckerorder))
         } else {
           return(ichimuraMaxPenalty)
         }
@@ -284,7 +344,7 @@ npindexbw.sibandwidth <-
 
         ## Next we define the sum of logs
 
-        sum.log.leave.one.out <- function(xdat,ydat,beta,h) {
+        sum.log.leave.one.out <- function(xdat,ydat,beta,h,ckertype,ckerorder) {
 
           ## Normalize beta_1 = 1 hence multiply X by c(1,beta)
           
@@ -299,7 +359,10 @@ npindexbw.sibandwidth <-
                         tydat=W,
                         weights=W,
                         leave.one.out=TRUE,
-                        bandwidth.divide=TRUE,bws=c(h))$ksum
+                        bandwidth.divide=TRUE,
+                        bws=c(h),
+                        ckertype=ckertype,
+                        ckerorder=ckerorder)$ksum
 
           denom <- tww[2,2,]
           denom[which(denom == 0.0)] <- kleinspadyFloor
@@ -321,7 +384,7 @@ npindexbw.sibandwidth <-
         ## return an infinite penalty for negative h
 
         if(h > 0) {
-          return(sum.log.leave.one.out(xdat,ydat,beta,h))
+          return(sum.log.leave.one.out(xdat=xdat,ydat=ydat,beta=beta,h=h,ckertype=ckertype,ckerorder=ckerorder))
         } else {
           ## No natural counterpart to var of y here, unlike Ichimura above...
           return(sqrt(.Machine$double.xmax))
@@ -344,7 +407,9 @@ npindexbw.sibandwidth <-
                            maxit=optim.maxit)
       } else if(bws$method == "kleinspady"){
         optim.fn <- kleinspady
-        optim.control <- c(reltol=optim.reltol,maxit=optim.maxit)
+        optim.control <- c(abstol=optim.abstol, ## added jracine jan 20 2011 (was in ichimura but not kleinspady)
+                           reltol=optim.reltol,
+                           maxit=optim.maxit)
       }
 
       for(i in 1:nmulti) {
@@ -433,6 +498,8 @@ npindexbw.sibandwidth <-
       bws$ifval <- best
       bws$numimp <- numimp
       bws$fval.vector <- fval.value
+      bws$ckertype <- ckertype
+      bws$ckerorder <- ckerorder
     }
     ## Return a list with beta (we append the restricted value of
     ## beta_1=1), the bandwidth h, the value of the objective function at
@@ -447,7 +514,7 @@ npindexbw.sibandwidth <-
     ## Restore seed
 
     if(exists.seed) assign(".Random.seed", save.seed, .GlobalEnv)
-    
+
     bws <- sibandwidth(beta = bws$beta,
                        h = bws$bw,
                        method = bws$method,
