@@ -29,7 +29,7 @@ npscoef.formula <-
     tmf <- bws$call[c(1,m)]
     tmf[[1]] <- as.name("model.frame")
     tmf[["formula"]] <- tt
-    umf <- tmf <- eval(tmf)
+    umf <- tmf <- eval(tmf, envir = environment(tt))
 
     tydat <- model.response(tmf)
     txdat <- tmf[, bws$chromoly[[2]], drop = FALSE]
